#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-08-02
# File  : unbound_type_stats.py

# based on the work of ruettimann[at]init7[dot]net

# https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml

# 2024-08-02: optimized answer_types and metrics creation
# 2024-08-18: added common and obsolete answer types
# 2024-08-27: removed duplicate answer types
#             added METRIC_TYPE and ADD-TOTAL options

from collections.abc import Mapping
from typing import Final
from cmk.gui.i18n import _

from cmk.gui.plugins.metrics.utils import (
    check_metrics,
    graph_info,
    metric_info,
    perfometer_info,
)

METRIC_TYPE: Final[str] = 'stack'  # can be stack or line
ADD_TOTAL: Final[bool] = True

answer_types_most_common: Mapping[str, str] = {
    'Total': '26/a',
    'Total common': '26/b',
    'Total obsolete': '16/b',
    'A': '11/a',
    'AAAA': '21/a',
    'ANY': '31/a',
    'CNAME': '41/a',
    'DNSKEY': '12/a',
    'DS': '22/a',
    'HINFO': '24/a',
    'HTTPS': '15/a',
    'MX': '32/a',
    'NAPTR': '42/a',
    'NS': '13/a',
    'PTR': '23/a',
    'RRSIG': '34/a',
    'SOA': '33/a',
    'SRV': '43/a',
    'SVCB': '44/a',
    'TXT': '14/a',
    'others': '36/a',
}

# https://en.wikipedia.org/wiki/List_of_DNS_record_types
answer_types_common: Mapping[str, str] = {
    'AFSDB': '11/a',
    'APL': '12/a',
    'CAA': '13/a',
    'CDNSKEY': '14/a',
    'CDS': '15/a',
    'CERT': '16/a',
    'CSYNC': '21/a',
    'DHCID': '22/a',
    'DLV': '23/a',
    'DNAME': '24/a',
    'EUI48': '25/a',
    'EUI64': '31/a',
    'HIP': '32/a',
    'IPSECKEY': '33/a',
    'KEY': '34/a',
    'KX': '35/a',
    'LOC': '36/a',
    'NSEC': '42/a',
    'NSEC3': '43/a',
    'NSEC3PARAM': '44/a',
    'OPENPGPKEY': '45/a',
    'RP': '46/a',
    'SIG': '11/b',
    'SMIMEA': '12/b',
    'SSHFP': '13/b',
    'TA': '14/b',
    'TKEY': '15/b',
    'TLSA': '16/b',
    'TSIG': '21/b',
    'URI': '22/b',
    'WALLET': '23/b',
    'ZONEMD': '24/b',
}

# https://en.wikipedia.org/wiki/List_of_DNS_record_types
answer_types_obsolete: Mapping[str, str] = {
    'MD': '11/b',
    'MF': '12/b',
    'MAILA': '13/b',
    'MB': '14/b',
    'MG': '15/b',
    'MR': '16/b',
    'MINFO': '21/b',
    'MAILB': '22/b',
    'WKS': '23/b',
    'NB': '24/b',
    'NBSTAT': '25/b',
    'NULL': '26/b',
    'A6': '31/b',
    'NXT': '32/b',
    'X25': '41/b',
    'ISDN': '42/b',
    'RT': '43/b',
    'NSAP': '44/b',
    'NSAP-PTR': '45/b',
    'PX': '46/b',
    'EID': '11/a',
    'NIMLOC': '12/a',
    'ATMA': '13/a',
    'SINK': '15/a',
    'GPOS': '16/a',
    'UINFO': '21/a',
    'UID': '22/a',
    'GID': '23/a',
    'UNSPEC': '24/a',
    'SPF': '25/a',
    'NINFO': '26/a',
    'RKEY': '31/a',
    'TALINK': '32/a',
    'NID': '33/qa',
    'L32': '34/a',
    'L64': '35/a',
    'LP': '36/a',
    'DOA': '41/a',
}

check_metrics["check_mk-unbound_type_stats"] = {}


def clean_metric(answer: str) -> str:
    return f'unbound_type_stats_{answer.replace(" ", "_").replace("-", "_")}'


for answer_type, color in answer_types_most_common.items():
    metric_info[clean_metric(answer_type)] = {
        'title': _(f'Rate of {answer_type} type stats'),
        'unit': '1/s',
        'color': color,
    }
    check_metrics["check_mk-unbound_type_stats"].update({clean_metric(answer_type): {"auto_graph": False}})

for answer_type, color in answer_types_common.items():
    metric_info[clean_metric(answer_type)] = {
        'title': _(f'Rate of {answer_type} type stats'),
        'unit': '1/s',
        'color': color,
    }
    check_metrics["check_mk-unbound_type_stats"].update({clean_metric(answer_type): {"auto_graph": False}})

for answer_type, color in answer_types_obsolete.items():
    metric_info[clean_metric(answer_type)] = {
        'title': _(f'Rate of {answer_type} type stats'),
        'unit': '1/s',
        'color': color,
    }
    check_metrics["check_mk-unbound_type_stats"].update({clean_metric(answer_type): {"auto_graph": False}})

graph_info['unbound_type_stats'] = {
    'title': _('Rate of type stats'),
    'metrics': [(
        clean_metric(answer), METRIC_TYPE) for answer in sorted(list(answer_types_most_common.keys())[3:], reverse=True)
    ],
    'optional_metrics': [clean_metric(answer) for answer in answer_types_most_common]
}

graph_info['unbound_type_stats_common'] = {
    'title': _('Rate of type stats (common)'),
    'metrics': [(
        clean_metric(answer), METRIC_TYPE) for answer in sorted(list(answer_types_common.keys()), reverse=True)
    ],
    'optional_metrics': [clean_metric(answer) for answer in answer_types_common]
}

graph_info['unbound_type_stats_obsolete'] = {
    'title': _('Rate of type stats (obsolete)'),
    'metrics': [(
        clean_metric(answer), METRIC_TYPE) for answer in sorted(list(answer_types_obsolete.keys()), reverse=True)
    ],
    'optional_metrics': [clean_metric(answer) for answer in answer_types_obsolete]
}

if ADD_TOTAL:
    graph_info['unbound_type_stats']['metrics'] = graph_info['unbound_type_stats']['metrics'] + [
        (f'unbound_type_stats_Total_obsolete', METRIC_TYPE),
        (f'unbound_type_stats_Total_common', METRIC_TYPE),
        (f'unbound_type_stats_Total', 'line'),
    ]
    graph_info['unbound_type_stats_common']['metrics'] = graph_info['unbound_type_stats_common']['metrics'] + [
        (f'unbound_type_stats_Total_common', 'line')]
    graph_info['unbound_type_stats_obsolete']['metrics'] = graph_info['unbound_type_stats_obsolete']['metrics'] + [
        (f'unbound_type_stats_Total_obsolete', 'line')]

perfometer_info.append(
    {
        'type': 'logarithmic',
        'metric': 'unbound_type_stats_Total',
        'half_value': 100.0,
        'exponent': 2,
    }
)
