#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-08-02
# File  : unbound_type_stats.py

# based on the work of ruettimann[at]init7[dot]net


from collections.abc import Sequence
from typing import Final

from cmk.gui.i18n import _
from cmk.gui.plugins.wato.utils import (
    CheckParameterRulespecWithoutItem,
    RulespecGroupCheckParametersApplications,
    rulespec_registry,
)

from cmk.gui.valuespec import (
    Alternative,
    Dictionary,
    FixedValue,
    Float,
    Percentage,
    Tuple,
)

# https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml
DNS_TYPES: Final[Sequence[str]] = [
    'A',
    'PTR',
    'AAAA',
    'ANY',
    'CNAME',
    'DNSKEY',
    'DS',
    'HINFO',
    'HTTPS',
    'MX',
    'NAPTR',
    'NS',
    'RRSIG',
    'SOA',
    'SRV',
    'SVCB',
    'TXT',
    # 'NULL',
    # 'TYPE0',
    # 'WKS',
    'unknown',
]

DNS_TYPES_CHOICES: Final[Sequence[str]] = [(answer, answer) for answer in DNS_TYPES]

_answer_levels = [
    (f'levels_upper_{dns_type}',
     Alternative(
         title=f'Upper levels for {dns_type} answers',
         show_alternative_title=True,
         elements=[
             Tuple(
                 elements=[
                     Float(title=_('Warning at'), unit=_('qps')),
                     Float(title=_('Critical at'), unit=_('qps')),
                 ],
                 title=_('Upper levels in qps'),
                 orientation='horizontal',
             ),
             Tuple(
                 elements=[
                     Percentage(title=_('Warning at'), unit=_('%')),
                     Percentage(title=_('Critical at'), unit=_('%')),
                     FixedValue(value='%', totext=''),  # needed to decide between both variants
                 ],
                 title=_('Upper levels in %'),
                 orientation='horizontal',
             ),
         ]))
    for dns_type in DNS_TYPES
]

_elements = _answer_levels
# _elements.append(
#     ('other_answer_types',
#      DualListChoice(
#          title=_('Other answer types'),
#          help=_('Selected answer types will be summarized under "other"'),
#          choices=DNS_TYPES_CHOICES,
#          rows=len(DNS_TYPES_CHOICES),
#          allow_empty=False,
#      ))
# )


def _parameter_valuespec_unbound_type_stats():
    return Dictionary(
        title=_('Unbound type stats'),
        elements=_elements,
    )


rulespec_registry.register(
    CheckParameterRulespecWithoutItem(
        check_group_name='unbound_type_stats',
        group=RulespecGroupCheckParametersApplications,
        match_type='dict',
        parameter_valuespec=_parameter_valuespec_unbound_type_stats,
        title=lambda: _('Unbound Type Stats'),
    )
)
